def _get_cert_data(filepath):
    "Returns the filepath content as bytes"
    with open(filepath, "rb") as fobj:
        return fobj.read()


def verify_files(inputfile, decrypted_output):
    # read both the files
    with open(inputfile) as f:
        original_text = f.read()

    with open(decrypted_output) as f:
        decrypted_text = f.read()
    assert original_text == decrypted_text



import os

import johnnycanencrypt.johnnycanencrypt as jce

from .conftest import BASE_TESTSDIR
from .utils import _get_cert_data, verify_files

DATA = "Kushal loves 🦀"


def test_encryption_of_multiple_keys_to_files(tmp_path):
    "Encrypt bytes to a file using multiple keys"
    output = tmp_path / "multiple-enc.asc"
    if os.path.exists(output):
        os.remove(output)
    certs = []
    for keyfilename in [
        BASE_TESTSDIR / "files/public.asc",
        BASE_TESTSDIR / "files/hellopublic.asc",
    ]:
        certs.append(_get_cert_data(keyfilename))
    jce.encrypt_bytes_to_file(
        certs,
        DATA.encode("utf-8"),
        output.as_posix().encode("utf-8"),
        armor=True,
    )
    assert os.path.exists(output)
    # Now let us decrypt it via first secret key
    with open(output, "rb") as f:
        enc = f.read()
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/hellosecret.asc"))
    result = jp.decrypt_bytes(enc, "redhat")
    assert DATA == result.decode("utf-8")

    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    result = jp.decrypt_bytes(enc, "redhat")
    assert DATA == result.decode("utf-8")


def test_encryption_of_multiple_keys_of_a_file(tmp_path):
    "Encrypt bytes to a file using multiple keys"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.pgp"
    decrypted_output = tmp_path / "text.txt"
    certs = []
    for keyfilename in [
        BASE_TESTSDIR / "files/public.asc",
        BASE_TESTSDIR / "files/hellopublic.asc",
    ]:
        certs.append(_get_cert_data(keyfilename))

    jce.encrypt_file_internal(
        certs,
        inputfile.as_posix().encode("utf-8"),
        output.as_posix().encode("utf-8"),
        armor=True,
    )
    assert os.path.exists(output)
    # Now let us decrypt it via second secret key
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/hellosecret.asc"))
    assert jp.decrypt_file(
        output.as_posix().encode("utf-8"), decrypted_output.as_posix().encode("utf-8"), "redhat"
    )
    verify_files(inputfile, decrypted_output)

    # Now remove it for next step
    os.remove(decrypted_output)

    # Via first secret key
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    assert jp.decrypt_file(
        output.as_posix().encode("utf-8"), decrypted_output.as_posix().encode("utf-8"), "redhat"
    )
    verify_files(inputfile, decrypted_output)


def test_encryption_of_multiple_keys_to_bytes():
    "Encrypt bytes using multiple keys"
    certs = []
    for keyfilename in [
        BASE_TESTSDIR / "files/public.asc",
        BASE_TESTSDIR / "files/hellopublic.asc",
    ]:
        certs.append(_get_cert_data(keyfilename))
    encrypted = jce.encrypt_bytes_to_bytes(
        certs,
        DATA.encode("utf-8"),
        armor=True,
    )
    # Now let us decrypt it via first secret key
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/hellosecret.asc"))
    result = jp.decrypt_bytes(encrypted, "redhat")
    assert DATA == result.decode("utf-8")

    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    result = jp.decrypt_bytes(encrypted, "redhat")
    assert DATA == result.decode("utf-8")


def test_encrypt_decrypt_bytes():
    "Tests raw bytes as output"
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    enc = jp.encrypt_bytes(DATA.encode("utf-8"))
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    result = jp.decrypt_bytes(enc, "redhat")
    assert DATA == result.decode("utf-8")


def test_encrypt_decrypt_bytes_armored():
    "Tests ascii-armored output"
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    enc = j.encrypt_bytes(DATA.encode("utf-8"), armor=True)
    assert enc.startswith(b"-----BEGIN PGP MESSAGE-----")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    result = jp.decrypt_bytes(enc, "redhat")
    assert DATA == result.decode("utf-8")


def test_encrypt_decrypt_files(tmp_path):
    "Tests encrypt/decrypt file in binary format"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.pgp"
    decrypted_output = tmp_path / "text.txt"

    # Now encrypt and then decrypt
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert j.encrypt_file(inputfile.as_posix().encode("utf-8"), output.as_posix().encode("utf-8"))
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    assert jp.decrypt_file(
        output.as_posix().encode("utf-8"), decrypted_output.as_posix().encode("utf-8"), "redhat"
    )

    verify_files(inputfile, decrypted_output)


def test_encrypt_decrypt_files_armored(tmp_path):
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.asc"
    decrypted_output = tmp_path / "text.txt"

    # Now encrypt and then decrypt
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert j.encrypt_file(
        inputfile.as_posix().encode("utf-8"), output.as_posix().encode("utf-8"), armor=True
    )
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    assert jp.decrypt_file(
        output.as_posix().encode("utf-8"), decrypted_output.as_posix().encode("utf-8"), "redhat"
    )

    with open(output) as f:
        line = f.readline().strip("\n")
        assert line == "-----BEGIN PGP MESSAGE-----"

    verify_files(inputfile, decrypted_output)


# The following data was generated while encrypting for 2 UID. Then we will try to decrypt
# using the second secret key.
# sequoia on  master [?] via 🦀 v1.46.0
# ❯ ./target/debug/sq encrypt -o double.asc --recipient-key-file ../rust/johnnycanencrypt/tests/files/store/hellopublic.asc --recipient-key-file ../rust/johnnycanencrypt/tests/files/store/public.asc msg.txt
# sequoia on  master [?] via 🦀 v1.46.0
# ❯ cp double.asc ../rust/johnnycanencrypt/tests/files/double_recipient.asc
# Test case for issue number #14
def test_decrypt_multiple_recipient_data():
    with open(BASE_TESTSDIR / "files/double_recipient.asc", "rb") as f:
        data = f.read()

    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    cleartext = jp.decrypt_bytes(data, "redhat")
    assert cleartext == b"Hello World! for 2.\n"


def test_encryption_of_multiple_keys_of_a_filehandler(tmp_path):
    "Encrypt bytes to an opened file using multiple keys"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted2.pgp"
    decrypted_output = tmp_path / "text2.txt"
    certs = []
    for keyfilename in [
        BASE_TESTSDIR / "files/public.asc",
        BASE_TESTSDIR / "files/hellopublic.asc",
    ]:
        certs.append(_get_cert_data(keyfilename))

    with open(inputfile, "rb") as fobj:
        jce.encrypt_filehandler_to_file(
            certs,
            fobj,
            output.as_posix().encode("utf-8"),
            armor=True,
        )
    assert os.path.exists(output)
    # Now let us decrypt it via second secret key
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/hellosecret.asc"))
    with open(output, "rb") as fobj:
        assert jp.decrypt_filehandler(fobj, decrypted_output.as_posix().encode("utf-8"), "redhat")
    verify_files(inputfile, decrypted_output)

    # Now remove it for next step
    os.remove(decrypted_output)

    # Via first secret key
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    with open(output, "rb") as fobj:
        assert jp.decrypt_filehandler(fobj, decrypted_output.as_posix().encode("utf-8"), "redhat")
    verify_files(inputfile, decrypted_output)



import os
import pytest

import johnnycanencrypt.johnnycanencrypt as rjce

from .conftest import BASE_TESTSDIR
from .utils import _get_cert_data

NISTP256_PUB = "ecdsa-sha2-nistp256 AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBEje+CqtHn9yp/vHBahLv01IeqS+6ZnD7ZQ87nAZZU6xPzTk5npdCq6q+mJBNsi/CNcV2H2Y1EuzsP1JylRyYqA= 123456\n"
NISTP384_PUB = "ecdsa-sha2-nistp384 AAAAE2VjZHNhLXNoYTItbmlzdHAzODQAAAAIbmlzdHAzODQAAABhBC2Xg9NOPD5HoHP3ee22gzhd2oAgRTx5EQFHuRS3jn/3MyJ8YYUeV8/i9+Xs7OTt6FsyVKDVCvelNqE6x1+aCKE0TblNCp9X9p7M8AegIobmEMwFbynSyYkK+FFGWGiUeQ== 123456\n"
NISTP521_PUB = "ecdsa-sha2-nistp521 AAAAE2VjZHNhLXNoYTItbmlzdHA1MjEAAAAIbmlzdHA1MjEAAACFBACwwC/dqTRtKGsovblIRCkgvfuElot4ma1Iiz5SsHpmPOoT/f/C+hbHkXzA+NO/IfJ4apWWYogydzHfsoZnMtL7cgBmPpOFRo+sOjlaqr9T6rRfznZqTqmb/EnOhmclvyOI+/i66kb7A+BybMh7jEtz4QQlsYbHDsxfepN7rJ/NZgMcVA== desk@phone\n"
NISTP521_PUB_NO_COMMENT = "ecdsa-sha2-nistp521 AAAAE2VjZHNhLXNoYTItbmlzdHA1MjEAAAAIbmlzdHA1MjEAAACFBACwwC/dqTRtKGsovblIRCkgvfuElot4ma1Iiz5SsHpmPOoT/f/C+hbHkXzA+NO/IfJ4apWWYogydzHfsoZnMtL7cgBmPpOFRo+sOjlaqr9T6rRfznZqTqmb/EnOhmclvyOI+/i66kb7A+BybMh7jEtz4QQlsYbHDsxfepN7rJ/NZgMcVA==\n"
RSA_PUB = "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQCqpNHfX9xOW41kl28wgeZHG/szYBldqflpG8HU8+OCZ6J5++Y4WmuHgl/W6ayrULtUWyKF1y7R0qcd8wf58PFwZMP+tAh3pij1vCSiFWYvhkq9b58smFHyHy8ZbpndKBexErpNygDsduy0ecw2wwqFDYn8EHs3tnuyT0Z99XQVScNzlqlLRAMxbLjyGurFSgqXjket9zkDbX6KhkryxiATGQql0inJqio2SkPHHYk2fQqlN4dXp/1oHsFrqGf247nDX3uNKnq7F7qTVbGmH3ehUzc9HqdRnUUFzWwTBn/VGU+zeUaEtBRtVewj/iqG0vKlo3LDm5Kp8LEbhGL88UlmBQRPISZYZ8Hm8lwkcOCnzXvf9gupxoXECqYChhbysMz66OqwAEplVHrFBqCFa0tIb6op+hVkHGuFXW8qlSTam/G0jLBJhRlOXduIrzn29mPhhVk11TQxqsVK9ji1RSG9yKaKxEjgS4z/M4GL0NrTUaVOdDXRDo1bfJHlsN5LSoBT0AwueQCgjieZRNAnQ9rPEPBM/5RGUq+vT//uzqOO9bE1iygixbkyRi6E+35wXqlobRDK8JEeGAKIdzA6NITqQXDHFPo1IsmrIbHagyOUSfH1QYRkG0kyIZBPcmjxjcv4UtjNHAVipWVdceS7FoVtnmPprwJf/hgQ7uIsHZ+DZw==\n"
EDDSA_PUB = "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIUC2eRpGLdX6BBudVLfZReqHpsHjnsn5Swlh6TECs2QZFQ== little@laptop\n"

def test_get_ssh_pubkey_nistp256():
    "To test ssh pubkey"
    nistp256_data = _get_cert_data(BASE_TESTSDIR / "files/nistp256.pub")
    pubkey = rjce.get_ssh_pubkey(nistp256_data, "123456")
    assert NISTP256_PUB == pubkey

def test_get_ssh_pubkey_nistp384():
    "To test ssh pubkey"
    nistp256_data = _get_cert_data(BASE_TESTSDIR / "files/nistp384.pub")
    pubkey = rjce.get_ssh_pubkey(nistp256_data, "123456")
    assert NISTP384_PUB == pubkey

def test_get_ssh_pubkey_nistp521():
    "To test ssh pubkey"
    nistp256_data = _get_cert_data(BASE_TESTSDIR / "files/nistp521.pub")
    pubkey = rjce.get_ssh_pubkey(nistp256_data, "desk@phone")
    assert NISTP521_PUB == pubkey

def test_get_ssh_pubkey_nistp521_no_comment():
    "To test ssh pubkey"
    nistp256_data = _get_cert_data(BASE_TESTSDIR / "files/nistp521.pub")
    pubkey = rjce.get_ssh_pubkey(nistp256_data, None)
    assert NISTP521_PUB_NO_COMMENT == pubkey

def test_no_authentication_key():
    rsa_data = _get_cert_data(BASE_TESTSDIR / "files/hellopublic.asc")
    with pytest.raises(rjce.CryptoError):
        pubkey = rjce.get_ssh_pubkey(rsa_data, None)

def test_get_ssh_pubkey_rsa():
    rsa_data = _get_cert_data(BASE_TESTSDIR / "files/store/kushal_updated_key.asc")
    pubkey = rjce.get_ssh_pubkey(rsa_data, None)
    assert RSA_PUB == pubkey

def test_get_ssh_pubkey_eddsa():
    rsa_data = _get_cert_data(BASE_TESTSDIR / "files/cv25519.pub")
    pubkey = rjce.get_ssh_pubkey(rsa_data, "little@laptop")
    assert EDDSA_PUB == pubkey





import os

import pytest

import johnnycanencrypt.johnnycanencrypt as jce

from .conftest import BASE_TESTSDIR
from .utils import _get_cert_data

DATA = "Kushal loves 🦀"


def test_sign_detached():
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    signature = j.sign_bytes_detached(DATA.encode("utf-8"), "redhat")
    assert signature


def test_sign_verify_bytes():
    "This will test a signed PGP message creation"
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    signed_data = j.sign_bytes(DATA.encode("utf-8"), "redhat", False).decode("utf-8")
    assert signed_data.endswith("-----END PGP MESSAGE-----\n")
    assert DATA not in signed_data
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert jp.verify_bytes(signed_data.encode("utf-8"))


def test_sign_cleartext():
    "This will test a signed cleartext message creation"
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    signed_data = j.sign_bytes(DATA.encode("utf-8"), "redhat", True).decode("utf-8")
    assert signed_data.startswith("-----BEGIN PGP SIGNED MESSAGE-----")
    assert DATA in signed_data
    assert signed_data.endswith("-----END PGP SIGNATURE-----\n")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert jp.verify_bytes(signed_data.encode("utf-8"))


def test_sign_verify_file_cleartext(tmp_path):
    "This will sign a file in cleartext"
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    output = (tmp_path / "sign.asc").as_posix()
    j.sign_file(
        (BASE_TESTSDIR / "files/text.txt").as_posix().encode(),
        output.encode("utf-8"),
        "redhat",
        True,
    )
    assert os.path.exists(output)
    with open(output) as fobj:
        data = fobj.read()
    assert data.startswith("-----BEGIN PGP SIGNED MESSAGE-----")
    assert "🦄🦄🦄" in data
    assert data.endswith("-----END PGP SIGNATURE-----\n")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert jp.verify_file(output.encode("utf-8"))


def test_sign_verify_file(tmp_path):
    "This will sign a file as a PGP message"
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    output = (tmp_path / "sign.asc").as_posix()
    j.sign_file(
        (BASE_TESTSDIR / "files/text.txt").as_posix().encode(),
        output.encode("utf-8"),
        "redhat",
        False,
    )
    assert os.path.exists(output)
    with open(output) as fobj:
        data = fobj.read()
    assert data.startswith("-----BEGIN PGP MESSAGE-----")
    assert "🦄🦄🦄" not in data
    assert data.endswith("-----END PGP MESSAGE-----\n")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert jp.verify_file(output.encode("utf-8"))


def test_sign_from_gpg_verify_file():
    "This will verify a signed message fro gpg"
    jp = jce.Johnny(
        _get_cert_data(BASE_TESTSDIR / "files/store/kushal_updated_key.asc")
    )
    assert jp.verify_file(str(BASE_TESTSDIR / "files/msg.txt.asc").encode("utf-8"))


def test_verify_signed_file(tmp_path):
    "This will verify a signed message from gpg and extract"
    jp = jce.Johnny(
        _get_cert_data(BASE_TESTSDIR / "files/store/kushal_updated_key.asc")
    )
    output = (tmp_path / "result.txt").as_posix()
    assert jp.verify_and_extract_file(
        str(BASE_TESTSDIR / "files/msg.txt.asc").encode("utf-8"), output.encode("utf-8")
    )

    # Now verify the text inside
    with open(output, "rb") as fobj:
        data = fobj.read()
    assert b"I \xe2\x9d\xa4\xef\xb8\x8f Anwesha.\n" == data


def test_verify_bytes_from_signed_message():
    "This will verify a signed message fro gpg"
    jp = jce.Johnny(
        _get_cert_data(BASE_TESTSDIR / "files/store/kushal_updated_key.asc")
    )
    with open(BASE_TESTSDIR / "files/msg.txt.asc", "rb") as fobj:
        data = fobj.read()
    assert b"I \xe2\x9d\xa4\xef\xb8\x8f Anwesha.\n" == jp.verify_and_extract_bytes(data)


def test_sign_from_different_key_file():
    "This will verify a signed message fro gpg"
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    with pytest.raises(jce.CryptoError):
        jp.verify_file(str(BASE_TESTSDIR / "files/msg.txt.asc").encode("utf-8"))


def test_verify_bytes_detached():
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    signature = j.sign_bytes_detached(DATA.encode("utf-8"), "redhat")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    assert jp.verify_bytes_detached(DATA.encode("utf-8"), signature.encode("utf-8"))


def test_verify_bytes_detached_must_fail():
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/secret.asc"))
    signature = j.sign_bytes_detached(DATA.encode("utf-8"), "redhat")
    jp = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    data2 = DATA + " "
    assert not jp.verify_bytes_detached(
        data2.encode("utf-8"), signature.encode("utf-8")
    )


def test_sign_detached_fail():
    j = jce.Johnny(_get_cert_data(BASE_TESTSDIR / "files/public.asc"))
    with pytest.raises(jce.CryptoError):
        signature = j.sign_bytes_detached(DATA.encode("utf-8"), "redhat")



import datetime
import shutil
import sqlite3

import pytest
import vcr

import johnnycanencrypt as jce
import johnnycanencrypt.johnnycanencrypt as rjce

from .conftest import BASE_TESTSDIR
from .utils import verify_files

DATA = "Kushal loves 🦀"


def test_correct_keystore_path():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")


def test_nonexisting_keystore_path():
    with pytest.raises(OSError):
        ks = jce.KeyStore(BASE_TESTSDIR / "files2/")


def test_str(tmp_path):
    ks = jce.KeyStore(tmp_path)
    assert str(ks) == f"<KeyStore dbpath={tmp_path.as_posix()}/jce.db>"


def test_no_such_key():
    with pytest.raises(jce.KeyNotFoundError):
        ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
        key = ks.get_key("A4F388BBB194925AE301F844C52B42177857DD79")


def test_create_primary_key_with_encryption(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    newkey = ks.create_key(
        "redhat",
        "test key42 <42@example.com>",
        jce.Cipher.RSA4k,
        whichkeys=1,
        can_primary_sign=True,
    )
    assert newkey.can_primary_sign == True

def test_key_cipher_details():
    saved = [('F4F388BBB194925AE301F844C52B42177857DD79', 'EdDSA', 256), ('102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F', 'EdDSA', 256), ('85B67F139D835FA56BA703DB5A7A1560D46ED4F6', 'ECDH', 256)]
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    result = rjce.get_key_cipher_details(key.keyvalue)
    assert saved == result


def test_keystore_lifecycle(tmp_path):
    # Now create a fresh db
    ks = jce.KeyStore(tmp_path.as_posix())
    newkey = ks.create_key("redhat", "test key1 <email@example.com>", jce.Cipher.RSA4k)
    # the default key must be of secret
    assert newkey.keytype == jce.KeyType.SECRET

    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/hellopublic.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
    # Now check the numbers of keys in the store
    assert (2, 2) == ks.details()

    ks.delete_key("F4F388BBB194925AE301F844C52B42177857DD79")
    assert (2, 1) == ks.details()

    # Now verify email cache
    key_via_fingerprint = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
    keys_via_emails = ks.get_keys(qvalue="kushaldas@gmail.com", qtype="email")
    assert len(keys_via_emails) == 1
    assert key_via_fingerprint == keys_via_emails[0]

    # Also verify that kushal's primary key can sign
    assert key_via_fingerprint.can_primary_sign == True

    # Now verify name cache
    key_via_fingerprint = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    keys_via_names = ks.get_keys(
        qvalue="Test User2 <random@example.com>", qtype="value"
    )
    assert len(keys_via_names) == 1
    assert key_via_fingerprint == keys_via_names[0]


def test_keystore_contains_key(tmp_path):
    "verifies __contains__ method for keystore"
    ks = jce.KeyStore(tmp_path.as_posix())
    keypath = BASE_TESTSDIR / "files/store/secret.asc"
    k = ks.import_key(keypath.as_posix())
    _, fingerprint, keytype, exp, ctime, othervalues = jce.parse_cert_file(
        keypath.as_posix()
    )

    # First only the fingerprint
    assert fingerprint in ks
    # Next the Key object
    assert k in ks
    # This should be false
    assert not "1111111" in ks


def test_keystore_details():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    assert (1, 2) == ks.details()


def test_keystore_keyids():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
    assert key.keyid == "D8219C8C43F6C5E1"


def test_keystore_get_via_keyids():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
    keys = ks.get_keys_by_keyid("FB82AA5D326DA75D")
    assert len(keys) == 1
    assert key == keys[0]


def test_keystore_key_uids():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
    assert "kushal@fedoraproject.org" == key.uids[0]["email"]
    assert "mail@kushaldas.in" == key.uids[-1]["email"]


def test_key_password_change(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    k = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
    k2 = ks.update_password(k, "redhat", "byebye")
    data = ks.sign_detached(k2, b"hello", "byebye")


def test_key_deletion(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
    k = ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/hellopublic.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/hellosecret.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
    assert (1, 2) == ks.details()

    ks.delete_key("F4F388BBB194925AE301F844C52B42177857DD79")
    assert (1, 1) == ks.details()

    # Now send in a Key object
    ks.delete_key(k)
    assert (0, 1) == ks.details()
    with pytest.raises(jce.KeyNotFoundError):
        ks.delete_key("11111")

    # Can not use any random data type
    with pytest.raises(TypeError):
        ks.delete_key(2441139)


def test_key_equality():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    assert key.fingerprint == "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"


def test_ks_update_expiry_time_for_subkeys(tmp_path):
    "Updates expiry time for a given subkey"
    ks = jce.KeyStore(tmp_path.as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/hellosecret.asc").as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())

    key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    subkeys = [
        "102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F",
    ]
    newexpiration = datetime.datetime(2050, 10, 25, 10)
    newkey = ks.update_expiry_in_subkeys(key, subkeys, newexpiration, "redhat")
    for _, skey in newkey.othervalues["subkeys"].items():
        if skey[0] == "102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F":
            date = skey[1]
            assert date.date() == datetime.date(2050, 10, 25)

    with pytest.raises(ValueError):
        newkey = ks.update_expiry_in_subkeys(key, subkeys, None, "redhat")


def test_ks_encrypt_decrypt_bytes():
    "Encrypts and decrypt some bytes"
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    encrypted = ks.encrypt(public_key, DATA)
    assert encrypted.startswith(b"-----BEGIN PGP MESSAGE-----\n")
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    decrypted_text = ks.decrypt(secret_key, encrypted, password="redhat").decode(
        "utf-8"
    )
    assert DATA == decrypted_text


def test_ks_encrypt_decrypt_bytes_multiple_recipients():
    "Encrypts and decrypt some bytes"
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    encrypted = ks.encrypt([key1, key2], DATA)
    assert encrypted.startswith(b"-----BEGIN PGP MESSAGE-----\n")
    secret_key1 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    decrypted_text = ks.decrypt(secret_key1, encrypted, password="redhat").decode(
        "utf-8"
    )
    assert DATA == decrypted_text
    secret_key2 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    decrypted_text = ks.decrypt(secret_key2, encrypted, password="redhat").decode(
        "utf-8"
    )
    assert DATA == decrypted_text


def test_ks_encrypt_decrypt_bytes_to_file(tmp_path):
    "Encrypts and decrypt some bytes"
    outputfile = tmp_path / "encrypted.asc"
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    assert ks.encrypt(secret_key, DATA, outputfile=outputfile.as_posix())
    with open(outputfile, "rb") as fobj:
        encrypted = fobj.read()
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    decrypted_text = ks.decrypt(secret_key, encrypted, password="redhat").decode(
        "utf-8"
    )
    assert DATA == decrypted_text


def test_ks_encrypt_decrypt_bytes_to_file_multiple_recipients(tmp_path):
    "Encrypts and decrypt some bytes"
    outputfile = tmp_path / "encrypted.asc"
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    assert ks.encrypt([key1, key2], DATA, outputfile=outputfile.as_posix())
    with open(outputfile, "rb") as fobj:
        encrypted = fobj.read()
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    decrypted_text = ks.decrypt(secret_key, encrypted, password="redhat").decode(
        "utf-8"
    )
    assert DATA == decrypted_text


def test_ks_encrypt_decrypt_file(tmp_path):
    "Encrypts and decrypt some bytes"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.pgp"
    decrypted_output = tmp_path / "text.txt"

    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    assert ks.encrypt_file(public_key, inputfile.as_posix(), output.as_posix())
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    ks.decrypt_file(
        secret_key, output.as_posix(), decrypted_output.as_posix(), password="redhat"
    )
    verify_files(inputfile, decrypted_output)


def test_ks_encrypt_decrypt_filehandler(tmp_path):
    "Encrypts and decrypt some bytes"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.pgp"
    decrypted_output = tmp_path / "text.txt"

    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    public_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    with open(inputfile, "rb") as fobj:
        assert ks.encrypt_file(public_key, fobj, output.as_posix())
    secret_key = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    with open(output, "rb") as fobj:
        ks.decrypt_file(
            secret_key, fobj, decrypted_output.as_posix(), password="redhat"
        )
    verify_files(inputfile, decrypted_output)


def test_ks_encrypt_decrypt_file_multiple_recipients(tmp_path):
    "Encrypts and decrypt some bytes"
    inputfile = BASE_TESTSDIR / "files/text.txt"
    output = tmp_path / "text-encrypted.pgp"
    decrypted_output = tmp_path / "text.txt"

    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    encrypted = ks.encrypt_file([key1, key2], inputfile.as_posix(), output.as_posix())
    secret_key1 = ks.get_key("F51C310E02DC1B7771E176D8A1C5C364EB5B9A20")
    ks.decrypt_file(
        secret_key1, output.as_posix(), decrypted_output.as_posix(), password="redhat"
    )
    verify_files(inputfile, decrypted_output)
    secret_key2 = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    ks.decrypt_file(
        secret_key2, output.as_posix(), decrypted_output.as_posix(), password="redhat"
    )
    verify_files(inputfile, decrypted_output)


def test_ks_sign_data():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
    signed = ks.sign_detached(key, "hello", "redhat")
    assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
    assert ks.verify(key, "hello", signed)


def test_ks_sign_data_fails():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
    signed = ks.sign_detached(key, "hello", "redhat")
    assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
    assert not ks.verify(key, "hello2", signed)


def test_ks_sign_verify_file_detached(tmp_path):
    inputfile = BASE_TESTSDIR / "files/text.txt"
    shutil.copy(inputfile, tmp_path.as_posix())
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
    file_to_be_signed = tmp_path / "text.txt"
    signed = ks.sign_file_detached(
        key, file_to_be_signed.as_posix(), "redhat", write=True
    )
    assert signed.startswith("-----BEGIN PGP SIGNATURE-----\n")
    assert ks.verify_file_detached(
        key, file_to_be_signed.as_posix(), file_to_be_signed.as_posix() + ".asc"
    )


def test_ks_userid_signing(tmp_path):
    # Now create a fresh db
    ks = jce.KeyStore(tmp_path.as_posix())
    k = ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
    t2 = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())

    # now let us sign the keys in kushal's uids
    k = ks.certify_key(
        t2,
        k,
        ["Kushal Das <kushaldas@gmail.com>", "Kushal Das <kushal@fedoraproject.org>"],
        jce.SignatureType.PersonaCertification,
        password="redhat",
    )
    # k now contains the new updated key
    for uid in k.uids:
        if (
            uid["value"] == "Kushal Das <kushaldas@gmail.com>"
            or uid["value"] == "Kushal Das <kushal@fedoraproject.org>"
        ):
            certs = uid["certifications"]
            # Only the new certification
            assert len(certs) == 1
            cert = certs[0]
            assert cert["certification_type"] == "persona"
            for data in cert["certification_list"]:
                if data[0] == "fingerprint":
                    assert data[1] == "F4F388BBB194925AE301F844C52B42177857DD79"
                if data[0] == "keyid":
                    assert data[1] == "C52B42177857DD79"
        else:
            assert len(uid["certifications"]) == 0


def test_ks_creation_expiration_time(tmp_path):
    """
    Tests via Kushal's key and a new key
    """
    # These two are known values from kushal
    etime = datetime.datetime(2020, 10, 16, 20, 53, 47)
    ctime = datetime.datetime(2017, 10, 17, 20, 53, 47)
    # First let us check from the file
    keypath = BASE_TESTSDIR / "files/store/pgp_keys.asc"
    ks = jce.KeyStore(tmp_path.as_posix())
    k = ks.import_key(keypath.as_posix())
    assert etime.date() == k.expirationtime.date()
    assert ctime.date() == k.creationtime.date()

    # now with a new key and creation time
    ctime = datetime.datetime(2010, 10, 10, 20, 53, 47)
    newk = ks.create_key(
        "redhat", "Another test key", ciphersuite=jce.Cipher.Cv25519, creation=ctime
    )
    assert ctime.date() == newk.creationtime.date()
    assert not newk.expirationtime

    # Now both creation and expirationtime for primary key
    ctime = datetime.datetime(2008, 10, 10, 20, 53, 47)
    etime = datetime.datetime(2025, 12, 15, 20, 53, 47)
    newk = ks.create_key(
        "redhat",
        "Another test key",
        creation=ctime,
        expiration=etime,
        can_primary_expire=True,
    )
    assert ctime.date() == newk.creationtime.date()
    assert etime.date() == newk.expirationtime.date()

    # Now both creation and expirationtime for subkeys
    ctime = datetime.datetime(2008, 10, 10, 20, 53, 47)
    etime = datetime.datetime(2029, 12, 15, 20, 53, 47)
    newk = ks.create_key(
        "redhat",
        "Test key with subkey expiration",
        creation=ctime,
        expiration=etime,
        subkeys_expiration=True,
    )
    assert ctime.date() == newk.creationtime.date()
    for skeyid, subkey in newk.othervalues["subkeys"].items():
        assert subkey[1].date() == etime.date()

    # Now only providing expirationtime for subkeys
    etime = datetime.datetime(2030, 6, 5, 20, 53, 47)
    newk = ks.create_key(
        "redhat",
        "Test key with subkey expiration",
        expiration=etime,
        subkeys_expiration=True,
    )
    assert datetime.datetime.now().date() == newk.creationtime.date()
    assert not newk.expirationtime
    for skeyid, subkey in newk.othervalues["subkeys"].items():
        assert subkey[1].date() == etime.date()

    # Now verify both subkeys and primary can expire
    etime = datetime.datetime(2030, 6, 5, 20, 53, 47)
    newk = ks.create_key(
        "redhat",
        "Test key with subkey expiration",
        expiration=etime,
        subkeys_expiration=True,
        can_primary_expire=True,
    )
    assert datetime.datetime.now().date() == newk.creationtime.date()
    assert etime.date() == newk.expirationtime.date()
    for skeyid, subkey in newk.othervalues["subkeys"].items():
        assert subkey[1].date() == etime.date()


def test_get_all_keys():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    keys = ks.get_all_keys()
    assert 3 == len(keys)
    # TODO: add more checks here in future


def test_get_pub_key():
    """Verifies that we export only the public key part from any key"""
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    fingerprint = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
    key = ks.get_key(fingerprint)
    # verify that the key is a secret
    assert key.keytype == jce.KeyType.SECRET

    # now get the public key
    pub_key = key.get_pub_key()
    assert pub_key.startswith("-----BEGIN PGP PUBLIC KEY BLOCK-----")


def test_add_userid(tmp_path):
    """Verifies that we can add uid to a cert"""
    ks = jce.KeyStore(tmp_path.as_posix())
    key = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
    # check that there is only one userid
    assert len(key.uids) == 1

    # now add a new userid
    key2 = ks.add_userid(key, "Off Spinner <spin@example.com>", "redhat")

    assert key2.fingerprint == key.fingerprint
    assert len(key2.uids) == 2
    assert key2.keytype == jce.KeyType.SECRET


def test_add_and_revoke_userid(tmp_path):
    """Verifies that we can add uid to a cert"""
    ks = jce.KeyStore(tmp_path.as_posix())
    key = ks.import_key((BASE_TESTSDIR / "files/store/secret.asc").as_posix())
    # check that there is only one userid
    assert len(key.uids) == 1

    # now add a new userid
    key2 = ks.add_userid(key, "Off Spinner <spin@example.com>", "redhat")

    assert key2.fingerprint == key.fingerprint
    assert len(key2.uids) == 2
    assert key2.keytype == jce.KeyType.SECRET
    # because at first all user ids are valid
    for uid in key2.uids:
        assert uid["revoked"] == False

    # now let us reove the new user id
    key3 = ks.revoke_userid(key2, "Off Spinner <spin@example.com>", "redhat")
    # verify the values
    assert key3.fingerprint == key.fingerprint
    assert len(key3.uids) == 2
    assert key3.keytype == jce.KeyType.SECRET
    for uid in key3.uids:
        if uid["value"] == "Off Spinner <spin@example.com>":
            assert uid["revoked"] == True
        else:
            assert uid["revoked"] == False


def test_add_userid_fails_for_public(tmp_path):
    """Verifies that adding uid to a public key fails"""
    ks = jce.KeyStore(tmp_path.as_posix())
    key = ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
    # verify that the key is a secret
    assert len(key.uids) == 1

    # now add a new userid
    with pytest.raises(ValueError):
        key2 = ks.add_userid(key, "Off Spinner <spin@example.com>", "redhat")


def test_update_subkey_expiry_time():
    "Updates the expirytime for a given subkey"
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    key = ks.get_key("F4F388BBB194925AE301F844C52B42177857DD79")
    fps = [
        "102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F",
    ]
    newkeyvalue = rjce.update_subkeys_expiry_in_cert(
        key.keyvalue, fps, 60 * 60 * 24, "redhat"
    )
    _, _, _, _, _, othervalues = rjce.parse_cert_bytes(newkeyvalue)
    tomorrow = datetime.date.today() + datetime.timedelta(days=1)
    for skey in othervalues["subkeys"]:
        if skey[1] == "102EBD23BD5D2D340FBBDE0ADFD1C55926648D2F":
            date = skey[3]
            assert date.date() == tomorrow


def test_same_key_import_error(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())
    with pytest.raises(jce.CryptoError):
        ks.import_key((BASE_TESTSDIR / "files/store/public.asc").as_posix())


def test_key_without_uid(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    k = ks.create_key("redhat")
    uids, fp, secret, et, ct, othervalues = jce.parse_cert_bytes(k.keyvalue)
    assert len(uids) == 0


def test_key_with_multiple_uids(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    uids = [
        "Kushal Das <kushaldas@gmail.com>",
        "kushal@freedom.press",
        "This is also Kushal",
    ]
    k = ks.create_key("redhat", uids)
    uids, fp, secret, et, ct, othervalues = jce.parse_cert_bytes(k.keyvalue)
    assert len(uids) == 3


def test_ks_upgrade(tmp_path):
    "tests db upgrade from an old db"
    shutil.copy(BASE_TESTSDIR / "files/store/oldjce.db", tmp_path / "jce.db")

    ks = jce.KeyStore(tmp_path.as_posix())
    con = sqlite3.connect(ks.dbpath)
    con.row_factory = sqlite3.Row
    # First we will check if this db schema is old or not
    with con:
        cursor = con.cursor()
        sql = "SELECT * from dbupgrade"
        cursor.execute(sql)
        fromdb = cursor.fetchone()
        assert fromdb["upgradedate"] == jce.DB_UPGRADE_DATE
    # TODO: Now verify the keys inside of the new db, in full.


def test_ks_upgrade_failure(tmp_path):
    "tests db upgrade failure from an old db because of existing file"
    shutil.copy(BASE_TESTSDIR / "files/store/oldjce.db", tmp_path / "jce.db")
    shutil.copy(BASE_TESTSDIR / "files/store/oldjce.db", tmp_path / "jce_upgrade.db")
    with pytest.raises(RuntimeError):
        ks = jce.KeyStore(tmp_path.as_posix())


def test_get_encrypted_for():
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store/")
    keyids = rjce.file_encrypted_for(
        (BASE_TESTSDIR / "files/double_recipient.asc").as_posix()
    )
    assert keyids == ["1CF980B8E69E112A", "5A7A1560D46ED4F6"]
    with open(BASE_TESTSDIR / "files/double_recipient.asc", "rb") as fobj:
        data = fobj.read()
    keyids = rjce.bytes_encrypted_for(data)
    assert keyids == ["1CF980B8E69E112A", "5A7A1560D46ED4F6"]


def test_available_subkeys_for_no_expiration():
    """Verifies that we export only the public key part from any key"""
    ks = jce.KeyStore(BASE_TESTSDIR / "files/store")
    fingerprint = "F51C310E02DC1B7771E176D8A1C5C364EB5B9A20"
    key = ks.get_key(fingerprint)
    e, s, a = key.available_subkeys()
    assert e == True
    assert s == True
    assert a == False


def test_available_subkeys_for_expired(tmp_path):
    """Verifies that we export only the public key part from any key"""
    ks = jce.KeyStore(tmp_path.as_posix())
    ks.import_key((BASE_TESTSDIR / "files/store/pgp_keys.asc").as_posix())
    key = ks.get_key("A85FF376759C994A8A1168D8D8219C8C43F6C5E1")
    e, s, a = key.available_subkeys()
    assert e == False
    assert s == False
    assert a == False


@vcr.use_cassette(
    (BASE_TESTSDIR / "files/test_fetch_key_by_fingerprint.yml").as_posix()
)
def test_fetch_key_by_fingerprint(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    key = ks.fetch_key_by_fingerprint("EF6E286DDA85EA2A4BA7DE684E2C6E8793298290")
    assert len(key.uids) == 1
    uid = key.uids[0]
    assert uid["email"] == "torbrowser@torproject.org"
    assert uid["name"] == "Tor Browser Developers"


@vcr.use_cassette(
    (BASE_TESTSDIR / "files/test_fetch_nonexistingkey_by_fingerprint.yml").as_posix()
)
def test_fetch_nonexistingkey_by_fingerprint(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    with pytest.raises(jce.KeyNotFoundError):
        key = ks.fetch_key_by_fingerprint("EF6E286DDA85EA2A4BA7DE684E2C6E8793298291")


@vcr.use_cassette((BASE_TESTSDIR / "files/test_fetch_key_by_email.yml").as_posix())
def test_fetch_key_by_email(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    key = ks.fetch_key_by_email("anwesha.srkr@gmail.com")
    assert len(key.uids) == 2
    uid = key.uids[0]
    assert uid["name"] == "Anwesha Das"
    assert key.fingerprint == "2871635BE3B4E5C04F02B848C353BFE051D06C33"


@vcr.use_cassette(
    (BASE_TESTSDIR / "files/test_fetch_nonexistingkey_by_email.yml").as_posix()
)
def test_fetch_nonexistingkey_by_email(tmp_path):
    ks = jce.KeyStore(tmp_path.as_posix())
    with pytest.raises(jce.KeyNotFoundError):
        ks.fetch_key_by_email("doesnotexists@kushaldas.in")



import datetime

import johnnycanencrypt.johnnycanencrypt as rustjce

from tests.conftest import BASE_TESTSDIR


def test_parse_cert_file():
    """Tests the rust implementation of the pgp key.

    Tests via Kushal's key and a new key
    """
    # These two are known values from kushal
    etime = datetime.datetime(2020, 10, 16, 20, 53, 47)
    ctime = datetime.datetime(2017, 10, 17, 20, 53, 47)
    # First let us check from the file
    keypath = BASE_TESTSDIR / "files/store/pgp_keys.asc"
    (
        uids,
        fingerprint,
        keytype,
        expirationtime,
        creationtime,
        othervalues,
    ) = rustjce.parse_cert_file(keypath.as_posix())
    assert etime.date() == expirationtime.date()
    assert ctime.date() == creationtime.date()
    assert othervalues["can_primary_sign"] == True


def test_parse_cert_bytes():
    """Tests the rust implementation of the pgp key.

    Tests via Kushal's key and a new key
    """
    # These two are known values from kushal
    etime = datetime.datetime(2020, 10, 16, 20, 53, 47)
    ctime = datetime.datetime(2017, 10, 17, 20, 53, 47)
    # First let us read from the file
    keypath = BASE_TESTSDIR / "files/store/pgp_keys.asc"
    with open(keypath, "rb") as fobj:
        data = fobj.read()

    (
        uids,
        fingerprint,
        keytype,
        expirationtime,
        creationtime,
        othervalues,
    ) = rustjce.parse_cert_bytes(data)
    assert etime.date() == expirationtime.date()
    assert ctime.date() == creationtime.date()


def test_merge_certs():
    """Tests the rust implementation of merging two OpenPGP keys.

    Tests via Kushal's old key, and the new key
    """
    # These two are known values from kushal
    ctime = datetime.datetime(2017, 10, 17, 20, 53, 47)
    # First let us read from the file
    keypath = BASE_TESTSDIR / "files/store/pgp_keys.asc"
    with open(keypath, "rb") as fobj:
        data = fobj.read()

    keypath = BASE_TESTSDIR / "files/store/kushal_updated_key.asc"
    with open(keypath, "rb") as fobj:
        newdata = fobj.read()

    newcert = rustjce.merge_keys(data, newdata, False)
    assert isinstance(newcert, bytes)

    (
        uids,
        fingerprint,
        keytype,
        expirationtime,
        creationtime,
        subkeys,
    ) = rustjce.parse_cert_bytes(newcert)

    assert ctime.date() == creationtime.date()
    assert not expirationtime


def test_no_primary_sign():
    keypath = (BASE_TESTSDIR / "files/store/secret.asc").as_posix()
    (
        uids,
        fingerprint,
        keytype,
        expirationtime,
        creationtime,
        othervalues,
    ) = rustjce.parse_cert_file(keypath)
    assert othervalues["can_primary_sign"] == False


def test_uid_certs():
    "To test certifications on user ids"
    keypath = (BASE_TESTSDIR / "files/store/kushal_updated_key.asc").as_posix()
    (
        uids,
        fingerprint,
        keytype,
        expirationtime,
        creationtime,
        othervalues,
    ) = rustjce.parse_cert_file(keypath)
    for uid in uids:
        if uid["value"] == "Kushal Das <kushaldas@gmail.com>":
            ctypes = {}
            assert len(uid["certifications"]) == 17
            for cert in uid["certifications"]:
                assert "creationtime" in cert
                assert "certification_type" in cert
                clist = cert["certification_list"]
                assert type(clist) == list
                for cvalue in clist:
                    if cvalue[0] == "fingerprint":
                        ctypes["fp"] = True
                    if cvalue[0] == "keyid":
                        ctypes["keyid"] = True
            # now verify that we have both the values in the certification_list
            assert ctypes["fp"]
            assert ctypes["keyid"]



from setuptools import setup
from setuptools_rust import Binding, RustExtension

setup(
    name="johnnycanencrypt",
    version="0.14.0",
    rust_extensions=[
        RustExtension("johnnycanencrypt.johnnycanencrypt", binding=Binding.PyO3)
    ],
    package_data={"johnnycanencrypt": ["py.typed", "johnnycanencrypt.pyi"]},
    packages=["johnnycanencrypt"],
    # rust extensions are not zip safe, just like C-extensions.
    zip_safe=False,
)
